module dee;

import pyd.pyd;
import std.math : PI;
import std.stdio;

class Vortex {
    cdouble m_position = 0.0 + 0.0i;
    double m_strength = 1.0;
    cdouble m_velocity = 0.0 + 0.0i;

    // Properties.
    cdouble position() {return m_position;}
    void position(cdouble pos) {m_position = pos;}
    double strength() {return m_strength;}
    void strength(double str) {m_strength = str;}
    cdouble velocity() {return m_velocity;}
    void velocity(cdouble pos) {m_velocity = pos;}

    this(cdouble pos = 0.0 + 0.0i, double str = 1.0)
    {
        m_position = pos;
        m_strength = str;
    }

    cdouble eval_velocity(cdouble pos)
    {
        return -1i*m_strength/(2.0*PI*(pos - m_position));
    }
}

class VortexManager {
    Vortex[] m_vortices;

    Vortex[] vortices() {return m_vortices;}
    void vortices(Vortex[] v) 
    {m_vortices.length = v.length; m_vortices[] = v;}

    cdouble eval_velocity(cdouble pos)
    {
        cdouble vel = 0.0 + 0.0i;
        foreach(Vortex x; m_vortices) {
            vel += x.eval_velocity(pos);
        }
        return vel;
    }

    void velocity()
    {
        cdouble vel;
        foreach(Vortex va; m_vortices){
            vel = 0.0 + 0.0i;
            foreach(Vortex vb; m_vortices){
                if (va != vb) {
                    vel += vb.eval_velocity(va.position());
                }
            }
            va.velocity(vel);
        }
    }

    void set(cdouble[] pos, double[] str)
    {
        assert(pos.length == str.length, "pos.length != str.length");
        m_vortices.length = pos.length;
        for(int i=0; i< pos.length; i++) {
            m_vortices[i] = new Vortex(pos[i], str[i]);
        }
    }

    void get(cdouble[] pos, double[] str, cdouble[] vel)
    {
        writefln("%s, %s, %s, %s", pos.length, str.length, vel.length,
                m_vortices.length);

       assert((pos.length == str.length) && 
               (str.length == vel.length) &&
               (pos.length == m_vortices.length));
       
        for(int i=0; i<m_vortices.length; i++) {
            pos[i] = m_vortices[i].position();
            str[i] = m_vortices[i].strength();
            vel[i] = m_vortices[i].velocity();
        }
    }
}



extern(C) void PydMain()
{
    module_init();
    wrap_class!(
            Vortex,
            Init!(void function(cdouble, double)),
            Def!(Vortex.eval_velocity),
            Property!(Vortex.position),
            Property!(Vortex.strength),
            Property!(Vortex.velocity)
            );
    wrap_class!(
            VortexManager,
            Def!(VortexManager.eval_velocity),
            Def!(VortexManager.velocity),
            Def!(VortexManager.set),
            Def!(VortexManager.get),
            Property!(VortexManager.vortices)
            );
}
