#include <complex>
#include <vector>
#include <iostream>
#include <cassert>

typedef std::complex<double> Point;

class Vortex {
public:
    Vortex(const Point &pos, const double str=1.0)
        : m_z(pos), m_s(str)
    { m_v = Point (0.0, 0.0); }

    Vortex(const Vortex &v)
        : m_z(v.m_z), m_v(v.m_v), m_s(v.m_s)
    {}
    virtual ~Vortex()
    {}

    Point position() const
    { return m_z; }
    Point velocity() const
    { return m_v; }
    double strength() const
    { return m_s; }

    void strength(const double str)
    { m_s = str; }
    void position(const Point &pos)
    { m_z = pos; }
    void velocity(const Point &v)
    { m_v = v; }

    virtual Point
    eval_velocity(const Point &pos) const
    {
        if (pos != m_z)
            return -Point(0.0, 1.0)*m_s/(2.0*M_PI*(pos - m_z));
        else
            return Point(0.0, 0.0);
    }

protected:
    Point m_z, m_v;
    double m_s;
};

class VortexManager {
public:
    ~VortexManager()
    {clear();}

    std::vector<Vortex* >& vortices()
    {return m_vortices;}

    void vortices(const std::vector<Vortex*> &v)
    {
        Vortex *vor, *vor1;
        clear();
        m_vortices.resize(v.size());
        for(std::size_t i=0; i<v.size(); ++i) {
            vor = v[i];
            vor1 = new Vortex(vor->position(),
                              vor->strength());
            vor1->velocity(vor->velocity());
            m_vortices[i] = vor1;
        }
    }

    void clear()
    {
        for(std::size_t i=0; i< m_vortices.size(); ++i) {
            delete m_vortices[i];
        }
        m_vortices.clear();
    }

    Point eval_velocity(const Point &pos)
    {
        Point vel(0.0, 0.0);
        for(std::size_t i=0; i< m_vortices.size(); ++i) {
            vel += m_vortices[i]->eval_velocity(pos);
        }
        return vel;
    }

    void velocity()
    {
        Point vel;
        Vortex *va, *vb;
        for(std::size_t i=0; i< m_vortices.size(); ++i){
            va = m_vortices[i];
            vel = Point(0.0, 0.0);
            for(std::size_t j=0; j< m_vortices.size(); ++j){
                vb = m_vortices[j];
                if (va != vb) {
                    vel += vb->eval_velocity(va->position());
                }
            }
            va->velocity(vel);
        }
    }

    void set(double* xin, const std::size_t xsize, 
             double* yin, const std::size_t ysize,
             double* strin, const std::size_t strsize
             )
    {
        assert((xsize == ysize) && (xsize == strsize));
        clear();
        m_vortices.resize(xsize);
        Vortex *v;
        for(std::size_t i=0; i< xsize; ++i) {
            v = new Vortex(Point(xin[i], yin[i]), strin[i]);
            m_vortices[i] = v;
        }
    }

protected:
    std::vector<Vortex*> m_vortices;
};

