import time
import sys
import numpy

try:
    import celerid
except ImportError:
    has_d = False
else:
    has_d = True

def make_pos(n=1000):
    z = numpy.zeros(n, complex)
    z.real = numpy.linspace(-1, 1, n)
    z.imag = numpy.linspace(-1, 1, n)
    str = numpy.ones(n, float)
    return z, str

def py(n=1000):
    import pure
    vm = pure.VortexManager()
    z, str = make_pos(n)
    vm.set(z, str)
    return vm

def cython(n=1000):
    import vortex 
    vm = vortex.VortexManager()
    z, str = make_pos(n)
    vm.set(z, str)
    return vm

def dee(n=1000):
    import dee
    vm = dee.VortexManager()
    z, str = make_pos(n)
    vm.set(z, str)
    return vm

def swig(n=1000):
    import simple
    vm = simple.VortexManager()
    z, str = make_pos(n)
    vm.set(z.real, z.imag, str)
    return vm

def timer(meth):
    t = time.time()
    meth()
    return time.time() - t

def test():
    if has_d:
        vm2 = dee(4000)
        print "dee(4000):", timer(vm2.velocity)
        print vm2.vortices[0].velocity, vm2.vortices[10].velocity

    vm3 = swig(4000)
    print "swig(4000):", timer(vm3.velocity)
    print vm3.vortices()[0].velocity(), vm3.vortices()[10].velocity()
    
    vm5 = cython(4000)
    v1, v2 = vm5.get(0), vm5.get(10)
    print "cython(4000):", timer(vm5.velocity)
    print complex(v1.u, v1.v), complex(v2.u, v2.v)

    vm = py(200)
    print "Pure Python(200):", timer(vm.velocity)
    #print vm.vortices[0].velocity, vm.vortices[10].velocity

def compare(of=sys.stdout):
    print >> of, "# N\tSWIG\tCython\tRatio"
    for i in range(1000, 11000, 1000):
        vm = swig(i)
        ts = timer(vm.velocity)
        vm = cython(i)
        tc = timer(vm.velocity)
        print >> of, "%d\t%.3f\t%.3f\t%.3f"%(i, ts, tc, tc/ts)
        of.flush()

if __name__ == '__main__':
    test()
    compare()
